use ::libc;

pub use crate::stddef_h::size_t;
pub use crate::stdlib::__uint8_t;
pub use crate::stdlib::u_int8_t;

pub use crate::squeezelite_h::buffer;
pub use crate::squeezelite_h::u8_t;
pub use crate::stdlib::__pthread_internal_list;
pub use crate::stdlib::__pthread_list_t;
pub use crate::stdlib::__pthread_mutex_s;
use crate::stdlib::free;
use crate::stdlib::malloc;
pub use crate::stdlib::pthread_mutex_destroy;
pub use crate::stdlib::pthread_mutex_init;
pub use crate::stdlib::pthread_mutex_lock;
pub use crate::stdlib::pthread_mutex_t;
pub use crate::stdlib::pthread_mutex_unlock;
pub use crate::stdlib::pthread_mutexattr_destroy;
pub use crate::stdlib::pthread_mutexattr_init;
pub use crate::stdlib::pthread_mutexattr_setprotocol;
pub use crate::stdlib::pthread_mutexattr_t;
pub use crate::stdlib::C2RustUnnamed_0;
pub use crate::stdlib::PTHREAD_PRIO_INHERIT;
pub use crate::stdlib::PTHREAD_PRIO_NONE;
pub use crate::stdlib::PTHREAD_PRIO_PROTECT;
/*
 *  Squeezelite - lightweight headless squeezebox emulator
 *
 *  (c) Adrian Smith 2012-2015, triode1@btinternet.com
 *      Ralph Irving 2015-2017, ralph_irving@hotmail.com
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
// fifo bufffers
// _* called with muxtex locked
#[inline]

unsafe extern "C" fn _buf_used(mut buf: *mut crate::squeezelite_h::buffer) -> libc::c_uint {
    return if (*buf).writep >= (*buf).readp {
        (*buf).writep.wrapping_offset_from((*buf).readp) as libc::c_long as libc::c_ulong
    } else {
        (*buf).size.wrapping_sub(
            (*buf).readp.wrapping_offset_from((*buf).writep) as libc::c_long as libc::c_ulong,
        )
    } as libc::c_uint;
}
#[no_mangle]

pub unsafe extern "C" fn _buf_space(mut buf: *mut crate::squeezelite_h::buffer) -> libc::c_uint {
    return (*buf)
        .size
        .wrapping_sub(_buf_used(buf) as libc::c_ulong)
        .wrapping_sub(1 as libc::c_int as libc::c_ulong) as libc::c_uint;
    // reduce by one as full same as empty otherwise
}
#[no_mangle]

pub unsafe extern "C" fn _buf_cont_read(
    mut buf: *mut crate::squeezelite_h::buffer,
) -> libc::c_uint {
    return if (*buf).writep >= (*buf).readp {
        (*buf).writep.wrapping_offset_from((*buf).readp) as libc::c_long
    } else {
        (*buf).wrap.wrapping_offset_from((*buf).readp) as libc::c_long
    } as libc::c_uint;
}
#[no_mangle]

pub unsafe extern "C" fn _buf_cont_write(
    mut buf: *mut crate::squeezelite_h::buffer,
) -> libc::c_uint {
    return if (*buf).writep >= (*buf).readp {
        (*buf).wrap.wrapping_offset_from((*buf).writep) as libc::c_long
    } else {
        (*buf).readp.wrapping_offset_from((*buf).writep) as libc::c_long
    } as libc::c_uint;
}
#[no_mangle]

pub unsafe extern "C" fn _buf_inc_readp(
    mut buf: *mut crate::squeezelite_h::buffer,
    mut by: libc::c_uint,
) {
    (*buf).readp = (*buf).readp.offset(by as isize);
    if (*buf).readp >= (*buf).wrap {
        (*buf).readp = (*buf).readp.offset(-((*buf).size as isize))
    };
}
#[no_mangle]

pub unsafe extern "C" fn _buf_inc_writep(
    mut buf: *mut crate::squeezelite_h::buffer,
    mut by: libc::c_uint,
) {
    (*buf).writep = (*buf).writep.offset(by as isize);
    if (*buf).writep >= (*buf).wrap {
        (*buf).writep = (*buf).writep.offset(-((*buf).size as isize))
    };
}
#[no_mangle]

pub unsafe extern "C" fn buf_flush(mut buf: *mut crate::squeezelite_h::buffer) {
    crate::stdlib::pthread_mutex_lock(&mut (*buf).mutex);
    (*buf).readp = (*buf).buf;
    (*buf).writep = (*buf).buf;
    crate::stdlib::pthread_mutex_unlock(&mut (*buf).mutex);
}
// adjust buffer to multiple of mod bytes so reading in multiple always wraps on frame boundary
#[no_mangle]

pub unsafe extern "C" fn buf_adjust(
    mut buf: *mut crate::squeezelite_h::buffer,
    mut mod_0: crate::stddef_h::size_t,
) {
    let mut size: crate::stddef_h::size_t = 0;
    crate::stdlib::pthread_mutex_lock(&mut (*buf).mutex);
    size =
        ((*buf).base_size.wrapping_div(mod_0) as libc::c_uint as libc::c_ulong).wrapping_mul(mod_0);
    (*buf).readp = (*buf).buf;
    (*buf).writep = (*buf).buf;
    (*buf).wrap = (*buf).buf.offset(size as isize);
    (*buf).size = size;
    crate::stdlib::pthread_mutex_unlock(&mut (*buf).mutex);
}
// called with mutex locked to resize, does not retain contents, reverts to original size if fails
#[no_mangle]

pub unsafe extern "C" fn _buf_resize(
    mut buf: *mut crate::squeezelite_h::buffer,
    mut size: crate::stddef_h::size_t,
) {
    crate::stdlib::free((*buf).buf as *mut libc::c_void);
    (*buf).buf = crate::stdlib::malloc(size) as *mut crate::squeezelite_h::u8_t;
    if (*buf).buf.is_null() {
        size = (*buf).size;
        (*buf).buf = crate::stdlib::malloc(size) as *mut crate::squeezelite_h::u8_t;
        if (*buf).buf.is_null() {
            size = 0 as libc::c_int as crate::stddef_h::size_t
        }
    }
    (*buf).readp = (*buf).buf;
    (*buf).writep = (*buf).buf;
    (*buf).wrap = (*buf).buf.offset(size as isize);
    (*buf).size = size;
    (*buf).base_size = size;
}
#[no_mangle]

pub unsafe extern "C" fn buf_init(
    mut buf: *mut crate::squeezelite_h::buffer,
    mut size: crate::stddef_h::size_t,
) {
    (*buf).buf = crate::stdlib::malloc(size) as *mut crate::squeezelite_h::u8_t;
    (*buf).readp = (*buf).buf;
    (*buf).writep = (*buf).buf;
    (*buf).wrap = (*buf).buf.offset(size as isize);
    (*buf).size = size;
    (*buf).base_size = size;
    let mut attr = crate::stdlib::pthread_mutexattr_t { __size: [0; 4] };
    crate::stdlib::pthread_mutexattr_init(&mut attr);
    crate::stdlib::pthread_mutexattr_setprotocol(
        &mut attr,
        crate::stdlib::PTHREAD_PRIO_INHERIT as libc::c_int,
    );
    crate::stdlib::pthread_mutex_init(&mut (*buf).mutex, &mut attr);
    crate::stdlib::pthread_mutexattr_destroy(&mut attr);
}
/*
 *  Squeezelite - lightweight headless squeezebox emulator
 *
 *  (c) Adrian Smith 2012-2015, triode1@btinternet.com
 *      Ralph Irving 2015-2017, ralph_irving@hotmail.com
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additions (c) Paul Hermann, 2015-2017 under the same license terms
 *   -Control of Raspberry pi GPIO for amplifier power
 *   -Launch script on power status change from LMS
 */
// make may define: PORTAUDIO, SELFPIPE, RESAMPLE, RESAMPLE_MP, VISEXPORT, GPIO, IR, DSD, LINKALL to influence build
// build detection
// dynamically loaded libraries at run time
// !LINKALL
// config options
// do not reduce as icy-meta max is 4080
/* SUN */
/* SUN */
// printf/scanf formats for u64_t
// logging
// utils.c (non logging)
// buffer.c
// _* called with mutex locked
#[no_mangle]

pub unsafe extern "C" fn buf_destroy(mut buf: *mut crate::squeezelite_h::buffer) {
    if !(*buf).buf.is_null() {
        crate::stdlib::free((*buf).buf as *mut libc::c_void);
        (*buf).buf = 0 as *mut crate::squeezelite_h::u8_t;
        (*buf).size = 0 as libc::c_int as crate::stddef_h::size_t;
        (*buf).base_size = 0 as libc::c_int as crate::stddef_h::size_t;
        crate::stdlib::pthread_mutex_destroy(&mut (*buf).mutex);
    };
}
